#ifndef OSCARDATA_H_INCLUDED
#define OSCARDATA_H_INCLUDED

#include <malloc.h>
#include <stdlib.h>
#include "oscarprotocol.h"

#define STARTING_OSCARDATA_SIZE 64
#define MRM_DEBUG_MEMORY
#define MRM_DEBUG_PRINTF

#define SWAP4( theLong ) \
  (unsigned long)                                   \
   ((((unsigned long)theLong & 0xff000000ul)>>24) | \
    (((unsigned long)theLong & 0x00ff0000ul)>>8) |  \
    (((unsigned long)theLong & 0x0000ff00ul)<<8) |  \
    (((unsigned long)theLong & 0x000000fful)<<24))

#define SWAP2( theShort )                      \
  (unsigned short)                             \
   ((((unsigned short)theShort & 0xff00)>>8) | \
    (((unsigned short)theShort & 0x00ff)<<8))


#ifndef MRM_DEBUG_PRINTF
#define debugf()
#else
extern int debugf( char *, ... );
#endif

#ifndef MRM_DEBUG_MEMORY
#define CHECKED_FREE( ptr )
  free( ptr )
#define CHECKED_MALLOC( theSize, newPtr ) \
  *((void **)(&(newPtr))) = malloc( theSize )
#define CHECKED_REALLOC( ptr, theSize, newPtr ) \
  *((void **)(&(newPtr))) = realloc( ptr, theSize )
#define CHECKED_CALLOC( theSize, newPtr )
  *((void **)(&(newPtr))) = calloc( theSize )
#else
#define CORRUPTION_REPORT( function, beforeAfter ) \
  debugf( "HEAP CORRUPTION DETECTED %s %s.\nFILE: %s\nLINE: %d\n\n", beforeAfter, function, __FILE__, __LINE__ ); \
  exit( 1 );
  
#define CHECKED_FREE( ptr ) \
{ \
  if ( _heapchk() != _HEAPOK ) { CORRUPTION_REPORT( "FREE", "PRIOR TO" ); } \
  free( ptr ); \
  if ( _heapchk() != _HEAPOK ) { CORRUPTION_REPORT( "FREE", "AFTER" ); } \
}
    
#define CHECKED_MALLOC( theSize, newPtr ) \
{ \
  if ( _heapchk() != _HEAPOK ) { CORRUPTION_REPORT( "MALLOC", "PRIOR TO" ); } \
  *((void **)(&(newPtr))) = malloc( theSize ); \
  if ( _heapchk() != _HEAPOK ) { CORRUPTION_REPORT( "MALLOC", "AFTER" ); } \
}

#define CHECKED_REALLOC( ptr, theSize, newPtr ) \
{ \
  if ( _heapchk() != _HEAPOK ) { CORRUPTION_REPORT( "REALLOC", "PRIOR TO" ); } \
  *((void **)(&(newPtr))) = realloc( (ptr), (theSize) ); \
  if ( _heapchk() != _HEAPOK ) { CORRUPTION_REPORT( "REALLOC", "AFTER" ); } \
}

#define CHECKED_CALLOC( theSize, newPtr ) \
{ \
  if ( _heapchk() != _HEAPOK ) { CORRUPTION_REPORT( "CALLOC", "PRIOR TO" ); } \
  *((void **)(&(newPtr))) = calloc( (theSize), 1 ); \
  if ( _heapchk() != _HEAPOK ) { CORRUPTION_REPORT( "CALLOC", "AFTER" ); } \
}

#endif

typedef struct
{
  unsigned char startByte;
  unsigned char channel;
  unsigned short sequence;
  unsigned short dataLen;
} FLAP;

typedef struct
{
  unsigned short family;
  unsigned short subType;
  char flags[2];
  unsigned long requestID;
} SNAC;

extern unsigned char *aim_encode_password( const char *password,
 int forceLength = 0 );

class OscarData
{
private:
  int length, position, shuttingDown, status;
  int *sequence;
  FLAP *ptr;
  
  void resizeAtLeast( int size );
  
public:
  OscarData();
  OscarData( int *sequenceNum );
  ~OscarData();
  
  void setSeqNumPointer( int *sequenceNum );
  
  void reset( void );
  void addFLAP( FLAP *theFLAP );
  void addFLAP( unsigned char channel );
  void addSNAC( SNAC *theSNAC );
  void addTLV( int type, int length, const void *value );
  void addTLV( int type, const char *value );
  void addData( const void *data, int dataLen );
  void tallyFLAPDataLen( void );
  void sendData( int socketHandle );
  void receiveData( int socketHandle );
  void printData( void );

  int parseTLV( unsigned char **TLVdata, unsigned short *tlvType );
  
  void shutdown( void );
  
  int getStatus( void );
  void getSNACtype( unsigned short *family, unsigned short *subType );

  // PREPARING DATA FOR SEND (oscarTx.cpp)
  // Main Oscar messages
  void prepareLoginData( char *userName, char *password );
  void prepareSignoffData( void );
  
  // BOS server messages
  void prepareAuthorizationData( AuthResponseData *theData );
  void prepareVersionsRequest( void );
  void prepareReqForSelfInformation( void );
  void prepareRateInfoRequest( void );
  void prepareCapabilitiesInfo( void );
  void prepareRequestSSIlimits( void );
  void prepareReqForSSI( void );
  void prepareSSIActivation( void );
  void prepareClientReady( const unsigned short *supportedFamilies );
  void prepareSetUserInformation( char *profileString, char *awayMsg );
  void prepareRequestICBMParams( void );
  void prepareSetICBMParams( int channel );
  void prepareRateInformationAck( userRateInformation *rateInfo );
  void prepareSetStatus( void );
  void prepareReqLocationLimits( void );
  void prepareReqBuddyManagementLimits( void );
  void prepareRequestPrivacyParams( void );
  void prepareRequestUserInfo( char *screenName, unsigned short infoType );
  void prepareRequestICBMparams( void );
  void prepareServerPauseAck( void );
  
  void prepareSetIdle( int idleTime );
  void sendInstantMessage( const char *toUser, const char *message );

  // PROCESSING RECEIVED DATA (oscarRx.cpp)
  // Main Oscar messages
  int isConnectionAck( void );
  int getAuthResponseData( AuthResponseData *theData );
  
  // BOS server messages
  int isHostReadyMessage( unsigned short **supportedFamilies );
  int isFamilyVersions( void );
  int isMOTD( void );
  int isRateInformation( userRateInformation **rateInfo );
  int isSSIlimits( void );
  int isServerPause( void );
  int isServerResume( void );
  int getUserInformation( int requestID, UserInformation *theInfo );
  int getSSIData( SSIData *theData );
  
  int instantMessageReceived( char **theUser, char **theMessage );
  
  unsigned short getAIMerror( void );
};

inline void OscarData :: shutdown( void )
{
  shuttingDown = 1;
}

inline void OscarData :: setSeqNumPointer( int *sequenceNum )
{
  sequence = sequenceNum;
}

#endif

